// dialogs.cpp: handles messages to dialogs


/*********************************************************************
soupe - Soph** Update Extension
(c) 2004-2008 Ulf Buermeyer - ulf@buermeyer.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*********************************************************************/


#include "stdafx.h"
#include "soupe.h"
#define MAX_LOADSTRING 100


// Message handler for options dialog
LRESULT CALLBACK Options(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{

	   case WM_INITDIALOG:
      {
         LPTSTR lpPath;

         // path to sav
         if (Reg_GetString(IDS_SAV_PATH, (LPTSTR *) &lpPath)) // path manually set in reg
         {
            SetDlgItemText(hDlg, IDC_SAVPATH, lpPath);
            free(lpPath);
         }
         else // use sophos path from reg
         {
         if (Reg_GetStringEx(HKEY_LOCAL_MACHINE, IDS_REG_SAV, IDS_SAV_PROGPATH, (LPTSTR *) &lpPath))
            {
               SetDlgItemText(hDlg, IDC_SAVPATH, lpPath);
               free(lpPath);
            }
         } // end else -> no path in reg
         

         // URL to download from
         if (Reg_GetString(IDS_IDE_URL, (LPTSTR *) &lpPath))
         {
            SetDlgItemText(hDlg, IDC_IDEURL, lpPath);
            free(lpPath);
         }
         else // use URL from stringtable
         {
            TCHAR szDefURL[STRINGSIZE];

            if(LoadString(GetModuleHandle(NULL), IDS_DEF_IDE_URL, szDefURL, sizeof(szDefURL)/sizeof(TCHAR)))
            {
               SetDlgItemText(hDlg, IDC_IDEURL, (LPTSTR)szDefURL);
            }
         } // end else -> no URL in reg


         // temp path to save to & decode in
         if (Reg_GetString(IDS_TEMP_PATH, (LPTSTR *) &lpPath))
         {
            SetDlgItemText(hDlg, IDC_TEMPPATH, lpPath);
            free(lpPath);
         }
         else // use URL from stringtable
         {
            TCHAR szDefPath[STRINGSIZE];
            TCHAR szTempPath[STRINGSIZE];

            if(LoadString(GetModuleHandle(NULL), IDS_DEF_TEMP_PATH, szDefPath, sizeof(szDefPath)/sizeof(TCHAR)))
            {
               if (Reg_GetStringEx(HKEY_LOCAL_MACHINE, IDS_REG_SAV, IDS_SAV_PROGPATH, (LPTSTR *) &lpPath))
               {
                  StringCbCopy(szTempPath, STRINGSIZE, lpPath);
                  StringCbCat(szTempPath, STRINGSIZE, TEXT("\\"));
                  StringCbCat(szTempPath, STRINGSIZE, szDefPath);
                  SetDlgItemText(hDlg, IDC_TEMPPATH, (LPTSTR)szTempPath);
                  free(lpPath);
               }
            }
         } // end else -> no URL in reg


		   return TRUE;
      } // WM_INITDIALOG


/////////////////////////////////////////////////////////////////


	   case WM_COMMAND:
		   switch(LOWORD(wParam))
         {
            case IDOK:
            {
               TCHAR szDlgText[STRINGSIZE];
               int iLen;

               // make sure our reg key is set
               if (!Reg_Init(IDS_REG_MAIN))
                  Reg_Install();

               iLen = GetDlgItemText(hDlg, IDC_SAVPATH, (LPTSTR)szDlgText, sizeof(szDlgText)/sizeof(TCHAR));
               if (iLen != 0)
               {
                  AddTrailing(szDlgText, STRINGSIZE, TEXT("\\"));
                  Reg_SetString(IDS_SAV_PATH, (LPTSTR) szDlgText);
               }


               iLen = GetDlgItemText(hDlg, IDC_TEMPPATH, (LPTSTR)szDlgText, sizeof(szDlgText)/sizeof(TCHAR));
               if (iLen != 0)
               {
                  AddTrailing(szDlgText, STRINGSIZE, TEXT("\\"));
                  // ad here: compare with SAV Dir !!
                  Reg_SetString(IDS_TEMP_PATH, (LPTSTR) szDlgText);
               }


               iLen = GetDlgItemText(hDlg, IDC_IDEURL, (LPTSTR)szDlgText, sizeof(szDlgText)/sizeof(TCHAR));
               if (iLen != 0)
               {
                  AddLeading(szDlgText, STRINGSIZE, TEXT("http://"));
                  Reg_SetString(IDS_IDE_URL, (LPTSTR) szDlgText);
               }
               else
               {
                  TCHAR szDefURL[STRINGSIZE];

                  if(LoadString(GetModuleHandle(NULL), IDS_DEF_IDE_URL, szDefURL, sizeof(szDefURL)/sizeof(TCHAR)))
                     Reg_SetString(IDS_IDE_URL, (LPTSTR) szDefURL);
                  PostMessage(GetParent(hDlg), WM_COMMAND, MAKEWPARAM(IDM_OPTIONS,0), 0);
                  EndDialog(hDlg, IDCANCEL);
               }

			      EndDialog(hDlg, LOWORD(wParam));
			      return TRUE;
		      }

            case IDCANCEL:
            {
               EndDialog(hDlg, LOWORD(wParam));
			      return TRUE;
            }

         } // switch command
		   break;

	} // switch (message to dialog)
	return FALSE;
}



// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
      TCHAR szLink[STRINGSIZE];
      RECT rc;

      GetDlgItemText(hDlg, IDC_LINK, szLink, sizeof(szLink)/sizeof(TCHAR));
      GetWindowRect(GetDlgItem(hDlg, IDC_LINK), &rc);
      
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}



// Message handler for about box.
LRESULT CALLBACK UpdateInterval(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
      {
         DWORD dwInterval;
         TCHAR szInterval[STRINGSIZE];

         dwInterval = Reg_GetDW(IDS_UPDATE_INTERVAL);
         StringCbPrintf((LPTSTR)szInterval, STRINGSIZE, TEXT("%u"), dwInterval);
         SetDlgItemText(hDlg, IDC_INTERVAL, (LPTSTR)szInterval);

         return TRUE;
      }
             
		

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK)
		{
         DWORD dwInterval;
         int iInterval;
         TCHAR szInterval[STRINGSIZE];

         if (!GetDlgItemText(hDlg, IDC_INTERVAL, szInterval, STRINGSIZE))
         {
            // can't get text
            EndDialog(hDlg, LOWORD(wParam));
            return TRUE;
         }
            
         iInterval = _ttoi(szInterval);
         if (iInterval < 0)
         {
            // invalid entry
            Error(TEXT("Die Zeitspanne darf nicht negativ sein. Bitte gltige Zahl eingeben."));
            return TRUE;
         }

         if (iInterval == 0)
         {
            // 0 -> turn off updates
            SetUpdateInterval(0);
            SetNextUpdate(0);
            Status(TEXT("Automatische Updates ausgeschaltet."));
            EndDialog(hDlg, LOWORD(wParam));
            return TRUE;
         }

         dwInterval = iInterval;

         if (dwInterval == GetUpdateInterval( ))
         {
            // no changes: don't reset timer
            EndDialog(hDlg, LOWORD(wParam));
            return TRUE;
         }
         
         // valid entry, new interval: set it
         SetUpdateInterval(dwInterval);
         SetNextUpdate(dwInterval);
         GetNextUpdateTime(szInterval, STRINGSIZE);
         Status(TEXT("Nchstes Update:|"));
         Status(szInterval);

			EndDialog(hDlg, LOWORD(wParam));
         return TRUE;
		} // end IDOK

      if (LOWORD(wParam) == IDCANCEL)
      {
         EndDialog(hDlg, LOWORD(wParam));
         return TRUE;
      }
      
		break;
	} // end WM_COMMAND
	return FALSE;
}

