// functions.cpp: helper functions


/*********************************************************************
soupe - Soph** Update Extension
(c) 2004-2008 Ulf Buermeyer - ulf@buermeyer.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*********************************************************************/


#include "stdafx.h"
#include "soupe.h"
#define MAX_LOADSTRING 100



BOOL AddTrailing(LPTSTR lpText, DWORD dwSize, LPTSTR lpTrail)
{
   DWORD dwLen;

   dwLen = (DWORD) _tcslen(lpText);
   if (!_tcsstr((LPTSTR)(lpText+dwLen-1), lpTrail))
   {
      StringCbCat(lpText, dwSize, lpTrail);
      return FALSE;
   }

   return TRUE;
}




BOOL AddLeading(LPTSTR lpText, DWORD dwSize, LPTSTR lpLead)
{
   LPTSTR lpBuf;

   if (lpText == _tcsstr((LPTSTR)(lpText), lpLead))
   {
      return TRUE;
   }
   else
   {
      lpBuf = (LPTSTR) malloc(dwSize);
      ZeroMemory(lpBuf, dwSize);
      StringCbCopy(lpBuf, dwSize, lpLead);
      StringCbCat(lpBuf, dwSize, lpText);
      StringCbCopy(lpText, dwSize, lpBuf);
      free(lpBuf);
      return FALSE;
   }

   return TRUE;
}



BOOL CheckDirectory(LPTSTR lpPath)
// return values: TRUE if exists, FALSE if not
{
   BOOL bResult;
   DWORD dwError;

   bResult = CreateDirectory(lpPath, NULL);
   if (bResult)
   {
      Status(TEXT("Erstelle Temp-Ordner|"));
      Status(lpPath);
      return TRUE;
   }
   
   dwError = GetLastError ( );
   if (dwError == ERROR_ALREADY_EXISTS)
      return TRUE;

   return FALSE;
}



LPTSTR FindLast(LPTSTR lpText, LPTSTR lpTrig)
{
   LPTSTR lpTemp;
   LPTSTR lpLast;

   lpLast = lpText;
   do
   {
      lpTemp = _tcsstr(lpLast, lpTrig);
      if (lpTemp)
         lpLast = lpTemp+1;
   } while (lpTemp);

   if ((lpLast == lpText) && (lpTemp == NULL)) // not found at all
      return NULL;

   return lpLast-1;
}




// returns pointer to first char of filename in a path
LPTSTR FileFromPath(LPTSTR lpPath)
{
   LPTSTR lpTemp;

   lpTemp = FindLast(lpPath, TEXT("\\"));

   // backslash not found but . in lpPath: shpuld be a file name...
   if ((lpTemp == NULL) && (_tcsstr(lpPath, TEXT("."))))
      return lpPath;

   // is there a . after last backslash?
   if ((lpTemp != NULL) && (_tcsstr(lpTemp+1, TEXT("."))))
      return (LPTSTR)(lpTemp+1);
   else // no backslash, no .
      return NULL;
}


// truncate a path including a filename to path only
BOOL TruncatePath(LPTSTR lpPath)
{
   LPTSTR lpLast;
   DWORD dwSize;

   // remember size
   dwSize = (DWORD) _tcslen(lpPath);

   // find last backslash
   lpLast = FindLast(lpPath, TEXT("\\"));
   if (!lpLast)
      return FALSE;
   
   // terminate path
   StringCbCopy(lpLast, dwSize, TEXT("\\"));

   return TRUE;
}


// restarts service named lpServ, log status output
BOOL RestartService(LPTSTR lpServ)
{
   SC_HANDLE schMan;
   SC_HANDLE schServ;
   SERVICE_STATUS ss;
   int iCount;

    
   schMan = OpenSCManager( 
      NULL,                    // local machine 
      NULL,                    // ServicesActive database 
      SC_MANAGER_ALL_ACCESS);  // full access rights 
 
   if (schMan == NULL)
   {
      StatusEx(TEXT("Kein Zugriff auf den Service-Manager: Fehler %u."), GetLastError());
      return FALSE;
   }

   schServ = OpenService( 
        schMan,          // SCM database 
        lpServ,          // service name
        SERVICE_ALL_ACCESS); 

   if (schServ == NULL)
   {
      StatusEx(TEXT("Kein Stop-Zugriff auf den Antivirus-Service: Fehler %u."), GetLastError());
      return FALSE;
   }


   // stop service
   iCount = 5;
   ControlService(schServ, SERVICE_CONTROL_STOP, &ss);
   Sleep(1000);
   QueryServiceStatus(schServ, &ss);

   while ((ss.dwCurrentState != SERVICE_STOPPED) && (iCount != 0))
   {
      ControlService(schServ, SERVICE_CONTROL_STOP, &ss);
      StatusEx(TEXT("Kein Stop-Zugriff auf den Antivirus-Service: Fehler %u; neuer Versuch..."), GetLastError());
      Sleep(1000);
      QueryServiceStatus(schServ, &ss);
      iCount--;
   }

   if (ss.dwCurrentState != SERVICE_STOPPED)
   {
      StatusEx(TEXT("Kein Stop-Zugriff auf den Antivirus-Service: Fehler %u; neuer Versuch..."), GetLastError());
      CloseServiceHandle(schServ);
      return FALSE;
   }

   //o.k., service stopped
   Status(TEXT("Service gestoppt; Startversuch (dies dauert einige Sekunden)..."));

   iCount = 5;
   StartService(schServ,0,NULL);

   // I/O normally takes at least that long
   Sleep(4000);

   QueryServiceStatus(schServ, &ss);
   while ((ss.dwCurrentState != SERVICE_RUNNING) && (iCount != 0))
   {
      StatusEx(TEXT("Kein Start-Zugriff auf den Antivirus-Service: Fehler %u; neuer Versuch..."), GetLastError());
      Sleep(2000);
      StartService(schServ,0,NULL);
      QueryServiceStatus(schServ, &ss);
      iCount--;
   }

   QueryServiceStatus(schServ, &ss);
   if (ss.dwCurrentState != SERVICE_RUNNING)
   {
      DWORD dwErr = GetLastError( );
      CloseServiceHandle(schServ);
      StatusEx(TEXT("Kein Start-Zugriff auf den Antivirus-Service: Fehler %u."), dwErr);
      ErrorEx(TEXT("Kein Start-Zugriff auf den Antivirus-Service: Fehler %u."), dwErr);
      return FALSE;
   }
   
   CloseServiceHandle(schServ);
   return TRUE;
}


// loads currently set update interval from reg
DWORD GetUpdateInterval(void)
{
   return Reg_GetDW(IDS_UPDATE_INTERVAL);
}

LPSTR GetAppVersionStringA(void)
{
   return GetAppVersionStringEx("A");
}

LPWSTR GetAppVersionStringW(void)
{
   return (LPWSTR) GetAppVersionStringEx("W");
}

// returns pointer to String Version x.x.x Build X (free mem after use!)
// param can be NULL, A or W (to choose ABSI, Unicode or auto detect output)
LPTSTR GetAppVersionStringEx(LPSTR lpCharset)
{
   TCHAR szName[STRINGSIZE];
   LPVOID lpBuf;
   VS_FIXEDFILEINFO *pFI;
   DWORD dwSize;
   DWORD dwNull;
   DWORD v1,v2,v3,v4;

   HANDLE hProcess;
   HMODULE hMod;
   DWORD dwNeeded;
   
   // find name of this exe file
   hProcess = GetCurrentProcess( );
   EnumProcessModules(hProcess, &hMod, sizeof(hMod), &dwNeeded);
   GetModuleBaseName(hProcess, hMod, szName, STRINGSIZE);
      
   // get size of biffer for VER-VERSIONINFO
   dwSize = GetFileVersionInfoSize((LPTSTR)szName, &dwNull); 

   // at least so much space needed for 2nd use as ver string buffer
   if (dwSize < STRINGSIZE)
      dwSize = STRINGSIZE;

   lpBuf = malloc(dwSize);
   if (!lpBuf)
      return NULL;

   if (!GetFileVersionInfo((LPTSTR)szName, 0, dwSize, lpBuf))
   {
      free(lpBuf);
      return NULL;
   }

   // get pointer to file versions in versioninfo
   if (!VerQueryValue(lpBuf,
                  TEXT("\\"),
                  (LPVOID *)&pFI,
                  (PUINT) &dwSize))
      {
      free(lpBuf);
      return NULL;
      }

   
   // extract
   v1 = HIWORD(pFI->dwFileVersionMS);
   v2 = LOWORD(pFI->dwFileVersionMS);
   v3 = HIWORD(pFI->dwFileVersionLS);
   v4 = LOWORD(pFI->dwFileVersionLS);

   // compose ver string
   // use A version of lstrcmp as param string to this function is always ANSI
   ZeroMemory(lpBuf, STRINGSIZE);
   if (lpCharset == NULL)
      StringCbPrintf((LPTSTR)lpBuf, STRINGSIZE, TEXT("Version %u.%u.%u Build %u"), v1,v2,v3,v4);
   if (0 == lstrcmpA(lpCharset, "W"))
      StringCbPrintfW((LPWSTR)lpBuf, STRINGSIZE, L"Version %u.%u.%u Build %u", v1,v2,v3,v4);
   if (0 == lstrcmpA(lpCharset, "A"))
      StringCbPrintfA((LPSTR)lpBuf, STRINGSIZE, "Version %u.%u.%u Build %u", v1,v2,v3,v4);
   if(strlen((LPTSTR) lpBuf) == 0)
   {
      free(lpBuf);
      lpBuf=0;
   }
      
   // exit
   return (LPTSTR)lpBuf;
}



BOOL GetLocalTimeString(LPTSTR lpTime, DWORD dwSize)
{
   SYSTEMTIME stUTC, stLocal;

   GetSystemTime(&stUTC);
   SystemTimeToTzSpecificLocalTime(NULL, &stUTC, &stLocal);

   // Build a string showing the date and time.
   StringCbPrintf(lpTime, dwSize, "%02d.%02d.%d  %02d:%02d:%02d:%04d  -   ",
        stLocal.wDay, stLocal.wMonth, stLocal.wYear,
        stLocal.wHour, stLocal.wMinute, stLocal.wSecond, stLocal.wMilliseconds);

   return TRUE;
}


DWORD SetUpdateInterval(DWORD dwNew)
{
   DWORD dwInterval;

   dwInterval = Reg_GetDW(IDS_UPDATE_INTERVAL);
   Reg_SetDW(IDS_UPDATE_INTERVAL, dwNew);

   return dwInterval;
}




BOOL GetNextUpdateTime(LPTSTR lpBuffer, DWORD dwSize)
{
   SYSTEMTIME st, stLocal;
   FILETIME ft;

   ft.dwLowDateTime = Reg_GetDW(IDS_UPDATELO);
   ft.dwHighDateTime = Reg_GetDW(IDS_UPDATEHI);

   if ((ft.dwHighDateTime == 0) || (ft.dwLowDateTime == 0))
   {
      StringCbCopy(lpBuffer, dwSize, TEXT("keine Update-Zeit eingestellt"));
      return FALSE;
   }
   
   FileTimeToSystemTime(&ft, &st);
   SystemTimeToTzSpecificLocalTime(NULL, &st, &stLocal);

   StringCbPrintf(lpBuffer, dwSize, "%02d.%02d.%d  %02d:%02d:%02d",
        stLocal.wDay, stLocal.wMonth, stLocal.wYear,
        stLocal.wHour, stLocal.wMinute, stLocal.wSecond);

return TRUE;
}



BOOL SetRunOnBoot(BOOL bRun)
{
   TCHAR szPath[STRINGSIZE];
   
   // quick & easy to erase...
   if (!bRun)
      return Reg_SetStringEx(HKEY_LOCAL_MACHINE, IDS_RUN_KEY, IDS_APP_TITLE, NULL);

   //else: find out path to exe file & set to reg
   if (!GetAppPath(szPath, STRINGSIZE)) return FALSE;
   StringCbCat(szPath, STRINGSIZE, TEXT(" -mini"));
   Reg_SetStringEx(HKEY_LOCAL_MACHINE, IDS_RUN_KEY, IDS_APP_TITLE, szPath);
   
   return TRUE;
}



BOOL GetAppPath(LPTSTR lpDest, DWORD dwSize)
{
   TCHAR szApp[STRINGSIZE];
   TCHAR szPath[STRINGSIZE];
   LPTSTR lpApp, lpTemp;

   HANDLE hProcess;
   HMODULE hMod;
   DWORD dwNeeded;

   // find name of this exe file
   hProcess = GetCurrentProcess( );
   EnumProcessModules(hProcess, &hMod, sizeof(hMod), &dwNeeded);
   GetModuleBaseName(hProcess, hMod, szApp, STRINGSIZE);
   if (_tcslen(szApp) == 0) return FALSE;

   // extract path from command line
   // get command line
   lpApp = GetCommandLine( );
   if (lpApp == NULL) return FALSE;
   StringCbCopy(szPath, STRINGSIZE, lpApp);

   // cut off everything after exe file
   lpTemp = _tcsstr(szPath, szApp);
   if (lpTemp == NULL) return FALSE;
   ZeroMemory(lpTemp + (_tcslen(szApp) * sizeof(TCHAR)), 2);
   
   // skip leading "
   lpTemp = _tcspbrk(szPath, TEXT("\"")); // find "
   if (lpTemp != NULL)
   {
      lpTemp += sizeof(TCHAR);
   }
   else
   {
      lpTemp = (LPTSTR)&szPath;
   }

   // check sizes & copy
   if (_tcslen(lpTemp) > dwSize) return FALSE;
   StringCbCopy(lpDest, dwSize, lpTemp);
   
return TRUE;
}


BOOL SetNextUpdate(DWORD dwInterval)
{
   SYSTEMTIME st;
   FILETIME ft;
   ULARGE_INTEGER uli, uli2;
   unsigned hyper iOneSecond;
   unsigned hyper iTicks;

   if (dwInterval == 0)
      dwInterval = GetUpdateInterval( );

   if (dwInterval == 0)
   {
      Reg_SetDW(IDS_UPDATELO, 0);
      Reg_SetDW(IDS_UPDATEHI, 0);
      return TRUE;
   }

   GetSystemTime(&st);
   SystemTimeToFileTime(&st, &ft);
   memcpy(&uli, &ft, sizeof(FILETIME));
      
   // get count for 1 second
   st.wSecond++;
   SystemTimeToFileTime(&st, &ft);
   memcpy(&uli2, &ft, sizeof(FILETIME));
   iOneSecond = uli2.QuadPart - uli.QuadPart;

   // add check delay
   iTicks = uli.QuadPart;
   iTicks += (((unsigned hyper)dwInterval) * iOneSecond * 60);
   uli.QuadPart = iTicks;

   // save but don't save zeroes as such
   if (uli.LowPart == 0) uli.LowPart++;
   if (uli.HighPart == 0) uli.HighPart++; // should never happen, we're not in 1601 anymore...
   Reg_SetDW(IDS_UPDATELO, uli.LowPart);
   Reg_SetDW(IDS_UPDATEHI, uli.HighPart);

   // checks
   memcpy(&ft, &uli, sizeof(FILETIME));
   FileTimeToSystemTime(&ft, &st);

   return TRUE;
}





// functions from MSDN: http://support.microsoft.com/default.aspx?scid=kb;EN-US;138813    

// (c) Microsoft Corporation

/*
 * AnsiToUnicode converts the ANSI string pszA to a Unicode string
 * and returns the Unicode string through ppszW. Space for the
 * the converted string is allocated by AnsiToUnicode.
 */ 


HRESULT __fastcall AnsiToUnicode(LPCSTR pszA, LPOLESTR* ppszW)
{

    ULONG cCharacters;
    DWORD dwError;

    // If input is null then just return the same.
    if (NULL == pszA)
    {
        *ppszW = NULL;
        return NOERROR;
    }

    // Determine number of wide characters to be allocated for the
    // Unicode string.
    cCharacters =  (ULONG) strlen(pszA)+1;

    // Use of the OLE allocator is required if the resultant Unicode
    // string will be passed to another COM component and if that
    // component will free it. Otherwise you can use your own allocator.
    *ppszW = (LPOLESTR) malloc(cCharacters*2);
    if (NULL == *ppszW)
        return E_OUTOFMEMORY;

    // Covert to Unicode.
    if (0 == MultiByteToWideChar(CP_ACP, 0, pszA, cCharacters,
                  *ppszW, cCharacters))
    {
        dwError = GetLastError();
        free(*ppszW);
        *ppszW = NULL;
        return HRESULT_FROM_WIN32(dwError);
    }

    return NOERROR;
}


/*
 * UnicodeToAnsi converts the Unicode string pszW to an ANSI string
 * and returns the ANSI string through ppszA. Space for the
 * the converted string is allocated by UnicodeToAnsi.
 */ 

HRESULT __fastcall UnicodeToAnsi(LPCOLESTR pszW, LPSTR* ppszA)
{

    ULONG cbAnsi, cCharacters;
    DWORD dwError;

    // If input is null then just return the same.
    if (pszW == NULL)
    {
        *ppszA = NULL;
        return NOERROR;
    }

    cCharacters = (ULONG) wcslen(pszW)+1;
    // Determine number of bytes to be allocated for ANSI string. An
    // ANSI string can have at most 2 bytes per character (for Double
    // Byte Character Strings.)
    cbAnsi = cCharacters*2;

    // Use of the OLE allocator is not required because the resultant
    // ANSI  string will never be passed to another COM component. You
    // can use your own allocator.
    *ppszA = (LPSTR) malloc(cbAnsi);
    if (NULL == *ppszA)
        return E_OUTOFMEMORY;

    // Convert to ANSI.
    if (0 == WideCharToMultiByte(CP_ACP, 0, pszW, cCharacters, *ppszA,
                  cbAnsi, NULL, NULL))
    {
        dwError = GetLastError();
        free(*ppszA);
        *ppszA = NULL;
        return HRESULT_FROM_WIN32(dwError);
    }
    return NOERROR;
}
