//***************************************************************
//
//			messages.cpp - procedures to handle window messages
//
//***************************************************************


/*********************************************************************
soupe - Soph** Update Extension
(c) 2004-2008 Ulf Buermeyer - ulf@buermeyer.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*********************************************************************/


#include "stdafx.h"
#include "soupe.h"

void Cls_OnSize(HWND hWnd, UINT state, int cx, int cy)
{
   if (state == SIZE_MINIMIZED)
      ShowWindow(hWnd, SW_HIDE);

   return;
}



void Cls_OnDestroy(HWND hWnd)
{
   NOTIFYICONDATA nid;

   // destroy tray icon
   nid.cbSize = sizeof(NOTIFYICONDATA);
   nid.hWnd = hWnd;
   nid.uID = UID;
   Shell_NotifyIcon(NIM_DELETE, &nid);
   
   PostQuitMessage(0);

   return;
}



void Cls_OnTimer(HWND hWnd, WPARAM wParam)
{
   SYSTEMTIME st;
   FILETIME ft;
   DWORD dwInterval;
   ULARGE_INTEGER uli, uli2;

   dwInterval = GetUpdateInterval( );
   if(dwInterval == 0) return;

   //Beep(2000, 50);

   // read time set
   uli.LowPart = Reg_GetDW(IDS_UPDATELO);
   uli.HighPart = Reg_GetDW(IDS_UPDATEHI);

   // get sys time
   GetSystemTime(&st);
   SystemTimeToFileTime(&st, &ft);
   memcpy(&uli2, &ft, sizeof(FILETIME));

   // compare
   if (uli2.QuadPart >= uli.QuadPart)
   {
      TCHAR szDate[STRINGSIZE];
      
      SetNextUpdate(dwInterval);
      StatusEx(TEXT("Automatisches Update wird gestartet & gleich neu programmiert: Updates alle %u Minuten,|"),dwInterval);
      Status(TEXT("nchstes Update um|"));
      GetNextUpdateTime(szDate, STRINGSIZE);
      Status(szDate);
      Status(TEXT("Ergebnis des automatischen Updates:"));
      PostMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDM_UPDATENOW,0), 0);
   }

   return;
}



BOOL Cls_OnCreate(HWND hWnd, LPCREATESTRUCT lpcs)
{
      TCHAR szKey[STRINGSIZE];
      LPTSTR lpVersion;
      INITCOMMONCONTROLSEX icc;
      RECT rc;
      HFONT hFont;
      LOGFONT lf;
      DWORD dwInterval;
      NOTIFYICONDATA nid;

      // load key string for reg access
      if (!LoadString(GetModuleHandle(NULL), IDS_REG_MAIN, (LPTSTR)szKey, sizeof(szKey)/sizeof(TCHAR)))
      {
         DestroyWindow(hWnd);
         return FALSE; // CreateWindow fails -> exit
      }

      // check if it exists in reg
      if (!Reg_Init(IDS_REG_MAIN))
      {
         Reg_Install( );
      }

      // init custom controls
		icc.dwSize = sizeof(icc);
		icc.dwICC = ICC_WIN95_CLASSES | 0x8000; //ICC_LINK_CLASS;
		InitCommonControlsEx(&icc);

      // init com
      CoInitialize(0);

      // create icon in systray
      nid.cbSize = sizeof(NOTIFYICONDATA);
      nid.hWnd = hWnd;
      nid.uID = UID;
      nid.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
      nid.uCallbackMessage = MYWM_NOTIFYICON;
      nid.hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_SOUPE));
      StringCbCopy(nid.szTip, NOTIFYICONDATA_V1_SIZE, szTitle);
      Shell_NotifyIcon(NIM_ADD, &nid);

      // create list view as status window
      GetClientRect(hWnd, &rc);
      hList = CreateWindow( TEXT("LISTBOX"), 0,
                  WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL, // | LBS_DISABLENOSCROLL,
                  0,0,
                  rc.right-rc.left,
                  rc.bottom-rc.top,
                  hWnd,
                  0,hInst,0);

      // create nice font & set it
      ZeroMemory(&lf, sizeof(lf));
      lf.lfCharSet = DEFAULT_CHARSET;
      lf.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
      hFont = CreateFontIndirect(&lf);
      SendMessage(hList, WM_SETFONT, (WPARAM)hFont, 0);

      // app is running: display start info
      Status(TEXT("Soupe|"));
      lpVersion = GetAppVersionStringEx(NULL);
      if (lpVersion)
      {
         StringCbCat(lpVersion, STRINGSIZE, TEXT("|"));
         Status(lpVersion);
         free(lpVersion);
      }
      Status(TEXT("gestartet."));
      ShowWindow(hList, SW_SHOWNORMAL);
      UpdateWindow(hList);

      // re-schedule automatic updates
      dwInterval = Reg_GetDW(IDS_UPDATE_INTERVAL);
      if (dwInterval)
      {
         TCHAR szDate[STRINGSIZE];

         SetNextUpdate(dwInterval);
         StatusEx(TEXT("Automatische Updates werden programmiert: Updates alle %u Minuten,|"),dwInterval);
         Status(TEXT("nchstes Update:|"));
         GetNextUpdateTime(szDate, STRINGSIZE);
         Status(szDate);
      }

      // auto-update on run if appropriate
      if (Reg_GetDW(IDS_UPDATE_ON_RUN) == 1)
      {
         PostMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDM_UPDATENOW,0), 0);
         Status(TEXT("Automatisches Update beim Programmstart wird ausgefhrt."));
      }

      // Install timer for auto update
      SetTimer(hWnd, IDM_UPDATENOW, 1000, NULL);

return TRUE; // success
}
