//***************************************************************
//
//			registry.cpp - procedures to handle registry settings
//
//***************************************************************



/*********************************************************************
soupe - Soph** Update Extension
(c) 2004-2008 Ulf Buermeyer - ulf@buermeyer.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*********************************************************************/


#include "stdafx.h"
#include "soupe.h"


//***************************************************************
//
//			Reg_Init - check that registry key is accessible
//
//
// 		in: 	LPTSTR to key to check
//			out:  TRUE or FALSE
//
//***************************************************************
BOOL Reg_Init(int iKey)
{
  	HKEY hKey;
  	LONG lRet;
   TCHAR szMainKey[STRINGSIZE];

   // main reg key
   if (!LoadString(GetModuleHandle(NULL), iKey, (LPTSTR)szMainKey, sizeof(szMainKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die Main String ID fehlt in stringtable."));
      return FALSE;
   }

   
   // open registry
  	lRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
               szMainKey,
               0, KEY_QUERY_VALUE, &hKey );

  	if(lRet != ERROR_SUCCESS)
            	{
               return FALSE;
               }

	// o.k. reg access finished
	RegCloseKey(hKey);

return TRUE;
}




//***************************************************************
//
//			Reg_Install - install default values on 1st run
//
//
// 		in: 	void
//			out:  TRUE or FALSE
//
//***************************************************************
BOOL Reg_Install(void)
{
LONG lRet;
HKEY hKey;
TCHAR szMainKey[STRINGSIZE];

   // main reg key
   if (!LoadString(GetModuleHandle(NULL), IDS_REG_MAIN, (LPTSTR)szMainKey, sizeof(szMainKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die Main String ID fehlt in stringtable."));
      return FALSE;
   }

  	// create key or open it if it exists already
  	lRet = RegCreateKey(HKEY_LOCAL_MACHINE,
               szMainKey,
               &hKey );
  	if(lRet != ERROR_SUCCESS)
            	{
               // even create does not work: error
               return FALSE;
               }

	// write defaults to reg
   Reg_SetDW(IDS_UPDATE_ON_RUN, 1);
   Reg_SetDW(IDS_UPDATE_INTERVAL, 30);
   Reg_SetDW(IDS_RUN_ON_BOOT, 1);
   SetRunOnBoot(TRUE);

	// o.k. reg access finished
	RegCloseKey(hKey);

return TRUE;
}


/***************************************************************
//
//			Reg_GetString(int, LPTSTR *)
//
//
// 		in: 	string resource code, pointer to LPTSTR
//			out:  TRUE or FALSE
//       !!    mem in pLP must be freed
//
//****************************************************************/
BOOL Reg_GetString(int iKey, LPTSTR *pLP)
{
   return Reg_GetStringEx(HKEY_LOCAL_MACHINE, IDS_REG_MAIN, iKey, pLP);
}


/***************************************************************
//
//			Reg_GetStringEx(int, int, LPTSTR *)
//
//
// 		in: 	string resource codes to main key / value; pointer to LPTSTR
//			out:  TRUE or FALSE
//
//****************************************************************/
BOOL Reg_GetStringEx(HKEY hRoot, int iMainKey, int iKey, LPTSTR *pLP)
{
HKEY hKey;
LONG lRet;
DWORD dwDataSize;
LPTSTR lpData;
TCHAR szMainKey[STRINGSIZE];
TCHAR szLoadKey[STRINGSIZE];

   // main reg key
   if (!LoadString(GetModuleHandle(NULL), iMainKey, (LPTSTR)szMainKey, sizeof(szMainKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die Main String ID fehlt in stringtable."));
      return FALSE;
   }

   // load key name to retrieve
   if (!LoadString(GetModuleHandle(NULL), iKey, (LPTSTR)szLoadKey, sizeof(szLoadKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die angegebene String ID fehlt in stringtable."));
      return FALSE;
   }
      
	// open registry
  	lRet = RegOpenKeyEx(hRoot,
               szMainKey,
               0, KEY_QUERY_VALUE, &hKey );

  	if(lRet != ERROR_SUCCESS)
      return FALSE;
      
   // first find out size of string to retrieve
  	lRet = RegQueryValueEx(hKey, szLoadKey, NULL, NULL,
               0, &dwDataSize);

            if(lRet != ERROR_SUCCESS || dwDataSize == 0)
             	{
               // nothing to retrieve
               return FALSE;
               }

	// o.k., key exists
   lpData = (LPTSTR) malloc(dwDataSize+sizeof(TCHAR));
   if (!lpData)
   	{
      Error(TEXT("Nicht gengend Arbeitspeicher verfgbar!"));
      return FALSE;
      }

   // actually retrieve
  	lRet = RegQueryValueEx(hKey, szLoadKey, NULL, NULL,
               (LPBYTE)lpData, &dwDataSize);

            if(lRet != ERROR_SUCCESS)
             	{
               // nothing retrieved
               return FALSE;
               }

   // o.k., success
   RegCloseKey(hKey);

   //zero-terminate
   *(lpData+(dwDataSize/sizeof(TCHAR))) = 0;

   *pLP = lpData;

   return TRUE; // success, pLP set
	}


/***************************************************************
//
//			Reg_GetDW(int)
//
//
// 		in: 	string resource codes to value name
//			out:  DWORD
//
//****************************************************************/
DWORD Reg_GetDW(int iKey)
{
   return Reg_GetDWEx(HKEY_LOCAL_MACHINE, IDS_REG_MAIN, iKey);
}



/***************************************************************
//
//			Reg_GetDWEx(HKEY, int, int)
//
//
// 		in: 	string resource codes to root key / main key / value name
//			out:  DWORD
//
//****************************************************************/
DWORD Reg_GetDWEx(HKEY hRoot, int iMainKey, int iKey)
{
HKEY hKey;
LONG lRet;
DWORD dwDataSize;
DWORD dwData;
TCHAR szMainKey[STRINGSIZE];
TCHAR szLoadKey[STRINGSIZE];

   // main reg key
   if (!LoadString(GetModuleHandle(NULL), iMainKey, (LPTSTR)szMainKey, sizeof(szMainKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die Main String ID fehlt in stringtable."));
      return 0;
   }

   // load key name to retrieve
   if (!LoadString(GetModuleHandle(NULL), iKey, (LPTSTR)szLoadKey, sizeof(szLoadKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die angegebene String ID fehlt in stringtable."));
      return 0;
   }
      
	// open registry
  	lRet = RegOpenKeyEx(hRoot,
               szMainKey,
               0, KEY_QUERY_VALUE, &hKey );

  	if(lRet != ERROR_SUCCESS)
      return 0;
      
   // first find out size of string to retrieve
  	lRet = RegQueryValueEx(hKey, szLoadKey, NULL, NULL,
               0, &dwDataSize);

            if(lRet != ERROR_SUCCESS || dwDataSize != sizeof(DWORD))
             	{
               // nothing to retrieve
               return 0;
               }

	
   // actually retrieve
  	lRet = RegQueryValueEx(hKey, szLoadKey, NULL, NULL,
               (LPBYTE)&dwData, &dwDataSize);

            if(lRet != ERROR_SUCCESS)
             	{
               // nothing retrieved
               return FALSE;
               }

   // o.k., success
   RegCloseKey(hKey);

   return dwData;
	}



/***************************************************************
//
//			Reg_SetDW(int, DWORD)
//
//
// 		in: 	string resource codes to value; value to set
//			out:  BOOL
//
//****************************************************************/
BOOL Reg_SetDW(int iKey, DWORD dwValue)
{
   return Reg_SetDWEx(HKEY_LOCAL_MACHINE, IDS_REG_MAIN, iKey, dwValue);
}



/***************************************************************
//
//			Reg_SetDWEx(HKEY, int, int, DWORD)
//
//
// 		in: 	string resource codes to root key, main key, value name; DWORD
//			out:  BOOL
//
//****************************************************************/
BOOL Reg_SetDWEx(HKEY hRoot, int iMainKey, int iKey, DWORD dwValue)
{
   HKEY hKey;
   LONG lRet;
   TCHAR szMainKey[STRINGSIZE];
   TCHAR szSaveKey[STRINGSIZE];

   // main reg key
   if (!LoadString(GetModuleHandle(NULL), iMainKey, (LPTSTR)szMainKey, sizeof(szMainKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die Main String ID fehlt in stringtable."));
      return FALSE;
   }

   // load key name to store in
   if (!LoadString(GetModuleHandle(NULL), iKey, (LPTSTR)szSaveKey, sizeof(szSaveKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die angegebene String ID fehlt in stringtable."));
      return FALSE;
   }

	// open registry
  	lRet = RegOpenKeyEx(hRoot,
               szMainKey,
               0, KEY_ALL_ACCESS, &hKey );

  	if(lRet != ERROR_SUCCESS)
            	{
               return FALSE;
               }

	// from this point on hKey contains a handle to our reg key
	lRet = RegSetValueEx(hKey,
				szSaveKey,
            0,
            REG_DWORD,
            (LPBYTE)&dwValue,
            sizeof(DWORD));
   
  	if(lRet != ERROR_SUCCESS)
            	{
               // store does not work: error
               return FALSE;
               }
               
   // o.k., success
   RegCloseKey(hKey);
   return TRUE;
}

/***************************************************************
//
//			Reg_SetStringEx(HKEY, int, int, LPTSTR)
//
//
// 		in: 	root key, string resource codes to main key + value name; LPTSTR
//			out:  BOOL
//
//****************************************************************/
BOOL Reg_SetStringEx(HKEY hRoot, int iMainKey, int iKey, LPTSTR lpData)
{
   HKEY hKey;
   LONG lRet;
   TCHAR szMainKey[STRINGSIZE];
   TCHAR szSaveKey[STRINGSIZE];

   // main reg key
   if (!LoadString(GetModuleHandle(NULL), iMainKey, (LPTSTR)szMainKey, sizeof(szMainKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die Main String ID fehlt in stringtable."));
      return FALSE;
   }

   // load key name to store in
   if (!LoadString(GetModuleHandle(NULL), iKey, (LPTSTR)szSaveKey, sizeof(szSaveKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die angegebene String ID fehlt in stringtable."));
      return FALSE;
   }

	// open registry
  	lRet = RegOpenKeyEx(hRoot,
               szMainKey,
               0, KEY_ALL_ACCESS, &hKey );


  	if(lRet != ERROR_SUCCESS)
            	{
               return FALSE;
               }

   // from this point on hKey contains a handle to our reg key

   if (lpData != NULL)
   {  
      lRet = RegSetValueEx(hKey,
				szSaveKey,
            0,
            REG_SZ,
            (LPBYTE)lpData,
            (DWORD)_tcslen(lpData)*sizeof(TCHAR));


  	   if(lRet != ERROR_SUCCESS)
         {
         // store does not work: error
         return FALSE;
         }
   }
   else
   {
      // null pointer -> erase key
      lRet = RegDeleteValue(hKey, szSaveKey);
      if(lRet != ERROR_SUCCESS)
         {
         // store does not work: error
         return FALSE;
         }
   }

               
   // o.k., success
   RegCloseKey(hKey);
   return TRUE;
}


//***************************************************************
//
//			Reg_SetString(int, LPTSTR)
//
//
// 		in: 	key code, pointer to LPTSTR
//			out:  TRUE or FALSE
//
//***************************************************************
BOOL Reg_SetString(int iKey, LPTSTR lpData)
{
HKEY hKey;
LONG lRet;
TCHAR szMainKey[STRINGSIZE];
TCHAR szSaveKey[STRINGSIZE];

   // main reg key
   if (!LoadString(GetModuleHandle(NULL), IDS_REG_MAIN, (LPTSTR)szMainKey, sizeof(szMainKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die Main String ID fehlt in stringtable."));
      return FALSE;
   }

   // load key name to store in
   if (!LoadString(GetModuleHandle(NULL), iKey, (LPTSTR)szSaveKey, sizeof(szSaveKey)/sizeof(TCHAR)))
   {
      Error(TEXT("Die angegebene String ID fehlt in stringtable."));
      return FALSE;
   }


	// open registry
  	lRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
               szMainKey,
               0, KEY_ALL_ACCESS, &hKey );


  	if(lRet != ERROR_SUCCESS)
            	{
               return FALSE;
               }

	// from this point on hKey contains a handle to our reg key
   if (lpData != NULL)
   {
      lRet = RegSetValueEx(hKey,
				szSaveKey,
            0,
            REG_SZ,
            (LPBYTE)lpData,
            (DWORD)_tcslen(lpData)*sizeof(TCHAR));
   
  	   if(lRet != ERROR_SUCCESS)  return FALSE;
   }
   else
   {
      // null pointer -> erase key
      lRet = RegDeleteValue(hKey, szSaveKey);
      if(lRet != ERROR_SUCCESS) return FALSE;
   }

               
   // o.k., success
   RegCloseKey(hKey);

return TRUE;
}
